/*
 * Quidditch
 * ---------
 * The starter code draws a yellow circle on the screen.  Every tick, there is a 5%
 * chance that the ball moves.  This demonstrates the animation technique, as well
 * as the random number generator.
 */

#include "GP142.h"
#include <math.h>
#include <time.h>
#include <stdlib.h>

#define TRUE 1
#define FALSE 0

#define BALL_SIZE 5		/* Ball radius in pixels */
#define BALL_BAMF 5		/* % chance of teleporting each tick */
/*
 * Lest you think that BAMF is an inappropriate name, in the X-Men comic book series,
 * there is a character named Nightcrawler that can teleport.  The sound he makes when
 * he teleports is BAMF.  Hence, BAMF has come to be synonymous with teleportation.
 */

/*
 * InitRandom
 * ----------
 * Input: n/a
 * Output: n/a
 * This function initializes the random-number generator using the
 * system clock.  You need to call this function before using any of
 * the GetRandomXXX functions, or the values returned by these functions
 * will be the same every time you run the program.  (This can be
 * useful when debugging.)
 */
void InitRandom(void) {
	time_t now;
	time(&now);
	srand(now);
}

/*
 * GetRandomInt
 * ------------
 * Input:
 * min = The minimum possible return value.
 * max = The maximum possible return value.
 * Output:
 * A random integer in the range [min, max]; each possible result is equally likely.
 * Every call to this function returns a random value in the range [min, max],
 * assuming that InitRandom has been called.  Otherwise, each time your
 * program runs, the output from GetRandomInt will be the same.  (Note that
 * while the program is running, successive calls to GetRandomInt will still
 * produce different values.)
 */
int GetRandomInt(int min, int max) {
	int r = rand();
	r = r % (max - min + 1);
	r = r + min;
	return r;
}

/*
 * GetRandomBool
 * -------------
 * Input:
 * percent = The probability with which the function will return TRUE.
 * Output:
 * TRUE with probability percent, FALSE otherwise.
 * Every call to this function returns TRUE or FALSE.  The actual result is
 * random assuming InitRandom has been called.  Otherwise, each time your program
 * runs, the output from GetRandomBool will be the same.
 */
int GetRandomBoolean(int percent) {
	return (GetRandomInt(0, 99) < percent);
}

/*
 * GetNumBashers
 * -------------
 * Input: n/a
 * Output:
 * An integer in the range [0, 2] depending on user input.
 * This function creates an input box that prompts the user for a value in
 * the range [0, 2].  If the user enters any other value (including a non-integer),
 * the function will try again.  Once the user enters a legal value, that value
 * is returned.
 */
int GetNumBashers(void) {
	char userInput[128];
	int result;
	while(TRUE) {
		GP142_gets("Enter the number of bashers on each side (0, 1 or 2):", userInput);
		if (!isdigit(userInput[0])) continue;
		result = atoi(userInput);
		if (result >= 0 && result <= 2) return result;
	}
}

/*
 * DrawBall
 * --------
 * Input:
 * center = The center of the ball to be drawn.
 * Output: n/a
 * This function draws a ball centered at the indicated point, in yellow, of size
 * BALL_SIZE.
 */
void DrawBall(GP142_point center) {
	GP142_clear();
	GP142_rectangleXY(BLACK, -GP142_XMAX, -GP142_YMAX, GP142_XMAX, GP142_YMAX, 0);
	GP142_circleP(YELLOW, center, BALL_SIZE);
}

/*
 * MoveBallAtRandom
 * ----------------
 * Input:
 * center = The center of the ball to be moved.
 * Output: n/a
 * This function only alters center with probability BALL_BAMF.  If the ball is to
 * be moved, a new location is chosen at random.
 */
void MoveBallAtRandom(GP142_point *center) {
	if (GetRandomBoolean(BALL_BAMF)) {
		center->x = GetRandomInt(-GP142_XMAX + BALL_SIZE, GP142_XMAX - BALL_SIZE);
		center->y = GetRandomInt(-GP142_YMAX + BALL_SIZE, GP142_YMAX - BALL_SIZE);
	}
}

int main(void)
{
	int stillRunning = TRUE;
	GP142_point center = {0, 0};
	GP142_event_t event;

	InitRandom();				/* Needed to generate random numbers. */
    GP142_open();				/* Starts the graphics package ... */
    GP142_logging(LOG_OFF);		/* without annoying log messages. */
	GP142_animate(ANI_RUN);

    while (stillRunning) {
		DrawBall(center);
        switch (GP142_await_eventP(&event)) {
			/* End the program. */
            case GP142_QUIT:
                stillRunning = FALSE;
                break;
   
			/* Ignore the event. */
            case GP142_MOUSE:
            case GP142_KBD:
                break;

			/* Every tick, the ball might move. */
            case GP142_PERIODIC:
				MoveBallAtRandom(&center);
                break;

            default:
                break;
        }
    }
        
    GP142_close();				/* Close the graphics window. */
    
    return 0;
}